import llm
from ollama import embeddings

MAX_LENGTH = 8192


@llm.hookimpl
def register_embedding_models(register):
    for model_id in (
        "mxbai-embed-large",
        "nomic-embed-text",
        "all-minilm",
        "bge-large",
        "bge-m3"
        
    ):
        register(OllamaEmbeddingModel(model_id))


class OllamaEmbeddingModel(llm.EmbeddingModel):
    def __init__(self, model_id):
        self.model_id = model_id
        self._model = None

    def embed_batch(self, texts):
        if self._model is None:
            self._model=embeddings
        # self.embeddings(model="mxbai-embed-large", prompt=d)
        results = [self._model(model=self.model_id, prompt=text[:MAX_LENGTH])["embedding"] for text in texts]
        return (list(map(float, result)) for result in results)



from click.testing import CliRunner
from llm.cli import cli
import llm
from llm.plugins import pm
import pytest
import os

IN_GITHUB_ACTIONS = os.getenv("GITHUB_ACTIONS") == "true"

def test_plugin_is_installed():
    names = [mod.__name__ for mod in pm.get_plugins()]
    assert "llm_embed_ollama" in names
    
@pytest.mark.skipif(IN_GITHUB_ACTIONS, reason="Not running on GitHub")
def test_ollama_embed_small():
    model = llm.get_embedding_model("all-minilm")
    floats = model.embed("hello world")
    assert len(floats) == 384
    assert all(isinstance(f, float) for f in floats)

@pytest.mark.skipif(IN_GITHUB_ACTIONS, reason="Not running on GitHub")
def test_ollama_embed_long_string():
    model = llm.get_embedding_model("all-minilm")
    outputs = list(model.embed_multi(["a" * 9000, "two"]))
    for floats in outputs:
        assert len(floats) == 384

@pytest.mark.skipif(IN_GITHUB_ACTIONS, reason="Not running on GitHub")
def test_ollama_embed_multi(tmpdir):
    db_path = str(tmpdir / "test.db")
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "embed-multi",
            "-m",
            "all-minilm",
            "test",
            "-",
            "-d",
            db_path,
        ],
        input='[{"id": "a", "text": "abc"}]',
        catch_exceptions=False,
    )
    assert result.exit_code == 0
